@@ -37,7 +37,7 @@ def erosion(input_tensor: torch.Tensor,
 
     # Adapt origin
     if not origin:
-        origin = (structuring_element.shape[0] // 2, structuring_element.shape[0] // 2)
+        origin = (structuring_element.shape[0] // 2, structuring_element.shape[1] // 2)
 
     # Fill border value if needed
     border_value = fill_border(border_value, 'erosion')
@@ -49,19 +49,7 @@ def erosion(input_tensor: torch.Tensor,
     if str(input_tensor.device) == 'cpu':
         # Pad input
         input_pad = pad_tensor(input_tensor, origin, structuring_element, border_value)
-
-        # Unfold the input
-        input_unfolded = input_pad
-        dim_shift = input_tensor.ndim - structuring_element.ndim
-        for dim in range(structuring_element.ndim):
-            input_unfolded = input_unfolded.unfold(dim_shift + dim, structuring_element.shape[dim], 1)
-
-        # Differences
-        result = input_unfolded - structuring_element
-
-        # Take the minimum
-        for dim in range(structuring_element.ndim):
-            result, _ = torch.min(result, dim=-1)
+        result = greyscale_morphology_cpp.greyscale_erosion(input_pad, structuring_element)
     else:
         if structuring_element.ndim == 2:
             # Pad input
@@ -126,7 +114,7 @@ def dilation(input_tensor: torch.Tensor,
 
     # Adapt origin
     if not origin:
-        origin = (structuring_element.shape[0] // 2, structuring_element.shape[0] // 2)
+        origin = (structuring_element.shape[0] // 2, structuring_element.shape[1] // 2)
 
     # Fill border value if needed
     border_value = fill_border(border_value, 'dilation')
@@ -137,20 +125,10 @@ def dilation(input_tensor: torch.Tensor,
     # Compute the dilation
     if str(input_tensor.device) == 'cpu':
         # Pad input
-        input_pad = pad_tensor(input_tensor, origin, structuring_element, border_value)
-
-        # Unfold the input
-        input_unfolded = input_pad
-        dim_shift = input_tensor.ndim - structuring_element.ndim
-        for dim in range(structuring_element.ndim):
-            input_unfolded = input_unfolded.unfold(dim + dim_shift, structuring_element.shape[dim], 1)
-
-        # Sums
-        result = input_unfolded + torch.flip(structuring_element, list(range(structuring_element.ndim)))
-
-        # Take the maximum
-        for dim in range(structuring_element.ndim):
-            result, _ = torch.max(result, dim=-1)
+        m, n = structuring_element.shape
+        o_m, o_n = origin
+        input_pad = pad_tensor(input_tensor, (m - (o_m + 1), n - (o_n + 1)), structuring_element, border_value)
+        result = greyscale_morphology_cpp.greyscale_dilation(input_pad, structuring_element)
     else:
         if structuring_element.ndim == 2:
             # Pad input
@@ -434,7 +412,7 @@ def erosion_dependent(input_tensor: torch.Tensor,
                       structuring_element: torch.Tensor,
                       origin: Optional[Union[tuple, List[int]]] = None,
                       border_value: Union[int, float, str] = 'geodesic'):
-    """ This type of erosion is needed when you want a structuring element to very along one axis.
+    """ This type of erosion is needed when you want a structuring element to vary along one axis.
 
         Parameters
         ----------
@@ -472,8 +450,8 @@ def erosion_dependent(input_tensor: torch.Tensor,
     input_tensor = convert_float(input_tensor)
 
     # Pad input
-    pad_list = [origin[0], structuring_element.shape[1] - origin[0] - 1,
-                origin[1], structuring_element.shape[2] - origin[1] - 1]
+    pad_list = [origin[1], structuring_element.shape[2] - origin[1] - 1,
+                origin[0], structuring_element.shape[1] - origin[0] - 1]
     input_pad = f.pad(input_tensor, pad_list, mode='constant', value=border_value)
 
     # Compute erosion
@@ -485,6 +463,61 @@ def erosion_dependent(input_tensor: torch.Tensor,
     return result
 
 
+def dilation_dependent(input_tensor: torch.Tensor,
+                       structuring_element: torch.Tensor,
+                       origin: Optional[Union[tuple, List[int]]] = None,
+                       border_value: Union[int, float, str] = 'geodesic'):
+    """ This type of dilation is needed when you want a structuring element to vary along one axis.
+
+        Parameters
+        ----------
+        :param input_tensor: torch.Tensor
+            The input tensor that you want to dilate. It should be a PyTorch tensor of 2 dimensions.
+        :param structuring_element: torch.Tensor
+            The structuring element to dilate. The structuring element should be a PyTorch tensor of 3 dimensions;
+            first dimension should coincide with first dimension of input_tensor and two other dimensions are the
+            shape of the structuring element.
+        :param origin: None, tuple, List[int]
+            The origin of the structuring element. Default to center of the structuring element.
+            Negative indexes are allowed. The origin will be the same for all the structuring elements.
+        :param border_value: int, float, str
+            The value used to pad the image in the border. Two options are allowed when a string is passed in parameter:
+            - 'geodesic': only points within the input are considered when taking the minimum.
+            - 'euclidean': extends naturally the image setting minus infinite value to the border.
+            Default value is 'geodesic'.
+
+        Outputs
+        -------
+        :return: torch.Tensor
+            The dilation dependent of the first axis as a PyTorch tensor of the same shape than the original input.
+    """
+    # Check parameters
+    check_parameters_dependent(input_tensor, structuring_element, origin, border_value)
+
+    # Adapt origin
+    if not origin:
+        origin = (structuring_element.shape[1] // 2, structuring_element.shape[2] // 2)
+
+    # Fill border value if needed
+    border_value = fill_border(border_value, 'dilation')
+
+    # Convert tensor to float if needed
+    input_tensor = convert_float(input_tensor)
+
+    # Pad input
+    pad_list = [origin[1], structuring_element.shape[2] - origin[1] - 1,
+                origin[0], structuring_element.shape[1] - origin[0] - 1]
+    input_pad = f.pad(input_tensor, pad_list, mode='constant', value=border_value)
+
+    # Compute dilation
+    if str(input_tensor.device) == 'cpu':
+        raise ValueError('Operation currently only implemented for GPU.')
+    else:
+        result = morphology_cuda.dilation_dependent(input_pad, structuring_element, BLOCK_SHAPE)
+
+    return result
+
+
 def partial_erosion(input_tensor: torch.Tensor,
                     structuring_element: torch.Tensor,
                     origin: Optional[Union[tuple, List[int]]] = None,
@@ -508,7 +541,7 @@ def partial_erosion(input_tensor: torch.Tensor,
 
     # Adapt origin
     if not origin:
-        origin = (structuring_element.shape[0] // 2, structuring_element.shape[0] // 2)
+        origin = (structuring_element.shape[0] // 2, structuring_element.shape[1] // 2)
 
     # Fill border value if needed
     border_value = fill_border(border_value, 'erosion')
@@ -517,7 +550,7 @@ def partial_erosion(input_tensor: torch.Tensor,
     input_tensor = convert_float(input_tensor)
 
     # Pad input
-    pad_list = [origin[0], structuring_element.shape[1] - origin[0] - 1]
+    pad_list = [origin[1], structuring_element.shape[1] - origin[1] - 1]
     input_pad = f.pad(input_tensor, pad_list, mode='constant', value=border_value)
 
     # Compute erosion
